/*
	LUSScreenSaver
	Copyright (C) 2003-2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSScreenSaverMisc.h"
#include "LUSScreenSaver.h"
#include "LUSScreenSaverInterface.h"
#include "LUSScreenSaverAuthInterface.h"
#include <stdlib.h>
#include <time.h>

using namespace LUSMisc;

LUSScreenSaver::LUSScreenSaver()
{
	DBG_LOG( "LUSScreenSaver::LUSScreenSaver()" );
	m_pAuthDlg = NULL;
	m_pSaver = NULL;
	m_nForceExecuted = 0;
	m_nDisableSuspend = 200;
	m_nTempMode = LUS_NOSAVE;
	srand( ( unsigned int ) time( NULL ) );
}

LUSScreenSaver::~LUSScreenSaver()
{
	DBG_LOG( "LUSScreenSaver::~LUSScreenSaver()" );
	m_nTempMode = LUS_NOSAVE;
	restore();
	AuthDel();
}

void LUSScreenSaver::restore()
{
	DBG_LOG( "LUSScreenSaver::restore()" );

	if ( m_nForceExecuted == 1 )
	{
		DBG_LOG( "LUSScreenSaver::restore() > Forced Mode. Nothing to do." );
		m_nForceExecuted ++;
		return;
	}
	else if ( m_nForceExecuted == 2 )
	{
		DBG_LOG( "LUSScreenSaver::restore() > Forced Mode. Reset intervals." );
		m_nForceExecuted = 0;
		QCopEnvelope* e = new QCopEnvelope( "QPE/System", "setScreenSaverIntervals(int,int,int)" );
		*e << -1 << -1 << -1;
		delete e;
	}

	SetLightLevel( GetLightLevel() );
	
	if ( m_pSaver )
	{
		LUSScreenSaverInterface* iface;
		if ( m_pSaver->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface ) == QS_OK )
		{
			iface->Stop();
			iface->release();
		}
		else
		{
			ExecOKDlg( "FATAL : Saver Can't Stop" );
		}
		m_pSaver->unload();
		delete m_pSaver;
		m_pSaver = NULL;
	
		if ( !m_pAuthDlg && IsLUSAuthInstalled() && IsPasswordEnable() )
		{
			DBG_LOG( "LUSScreenSaver::restore() > Do Authentication" );
			LUSScreenSaverAuthInterface* iface_auth;
			m_pAuthDlg = new QLibrary( "lusauth", QLibrary::Immediately );
			if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver,
											(QUnknownInterface**)&iface_auth ) == QS_OK )
			{
				iface_auth->Start( rand() );
			}
			else
			{
				DBG_LOG( "LUSScreenSaver::restore() > LUSAuth quetyInterface() failed" );
				delete m_pAuthDlg;
				m_pAuthDlg = NULL;
			}
		}
		else
		{
			RestartKeyHelper();
		}
	}

	m_nTempMode = LUS_NOSAVE;
}

bool LUSScreenSaver::save( int level )
{
	DBG_LOG( QString( "LUSScreenSaver::save( level : %1 )" ).arg( level ).latin1() );

	bool bRet = false;

	if ( m_nForceExecuted == 1 )
	{
		DBG_LOG( "save() > Forced Mode" );
		{
			/*
			QCopEnvelope* e = new QCopEnvelope( "QPE/System", "setScreenSaverIntervals(int,int,int)" );
			*e << -1 << -1 << -1;
			delete e;
			*/
			// QWSServer::setScreenSaverInterval( GetLUSInterval() );
		}
		DBG_LOG( "save() > Do ScreenSaver" );
		bool bRet = ShowScreenSaver();
		if ( bRet )
		{
			m_nTempMode = LUS_SCREENSAVING;
		}
		// m_bForceExecuted = FALSE;
		return bRet;
	}

	switch( level )
	{
	case 0:
		{
			if ( ( m_nDisableSuspend >= 0
				&& m_nDisableSuspend <= 0 )
				|| 
				IsSpecialAppExists( LUS_SCREENSAVING ) )
			{
				DBG_LOG( "save() > LEVEL0 intercepted" );
				bRet = true;
				break;
			}
			if ( WhichIsFirst( LUS_SCREENSAVING, LUS_LIGHTOFF, LUS_SUSPEND )
					== LUS_SCREENSAVING )
			{
				if ( IsACOnline() )
				{
					DBG_LOG( "save() > Do ScreenSaver" );
					bRet = ShowScreenSaver();
					if ( bRet )
					{
						m_nTempMode = LUS_SCREENSAVING;
					}
					break;
				}
				else
				{
					DBG_LOG( "save() > Do Dimmer" );
					SetLightLevel( GetDimLevel() );
					bRet = true;
					m_nTempMode = LUS_SCREENSAVING;
					break;
				}
			}
		}
		DBG_LOG( "save() > judgement : up saving level to 1" );
	case 1:
		{
			if ( ( m_nDisableSuspend >= 0
				&& m_nDisableSuspend <= 1 )
				|| 
				IsSpecialAppExists( LUS_SCREENSAVING ) )
			{
				DBG_LOG( "save() > LEVEL1 intercepted" );
				bRet = true;
				break;
			}
			if ( WhichIsFirst( LUS_LIGHTOFF, LUS_SUSPEND ) == LUS_LIGHTOFF
					&& m_nTempMode != LUS_LIGHTOFF )
			{
				DBG_LOG( "save() > Do LightOff" );
				SetLightLevel( 0 );
				bRet = TRUE;
				m_nTempMode = LUS_LIGHTOFF;
				break;
			}
		}
		DBG_LOG( "save() > judgement : up saving level to 2" );
	case 2:
		{
			if ( ( m_nDisableSuspend >= 0
				&& m_nDisableSuspend <= 2 )
				|| 
				IsSpecialAppExists( LUS_SUSPEND ) )
			{
				DBG_LOG( "save() > LEVEL2 intercepted" );
				bRet = true;
				break;
			}
			{
				DBG_LOG( "save() > Do suspend" );
				m_nTempMode = LUS_NOSAVE;
				Global::execute( "suspend" );
				bRet = true;
				break;
			}
		}
	default:
		break;
	}
	return bRet;
}

bool LUSScreenSaver::ShowScreenSaver()
{
	DBG_LOG( "LUSScreenSaver::ShowScreenSaver()" );
	if ( m_pSaver )
	{
		return true;
	}
	
	QString sSaverName = GetSaverName();
	if ( sSaverName.isEmpty() )
	{
		return false;
	}

	QString sSaverPath = AbsSaverPath( sSaverName );

	if ( !QFile::exists( sSaverPath ) )
	{
		ExecOKDlg( QString("\"%1\" Not Found" ).arg( sSaverName ) );
		return false;
	}

	DBG_LOG( "LUSScreenSaver::ShowScreenSaver() > Module loading start" );
	m_pSaver = new QLibrary( sSaverPath, QLibrary::Immediately );

    LUSScreenSaverInterface* iface;
    if ( m_pSaver->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface ) != QS_OK )
	{
		delete m_pSaver;
		m_pSaver = NULL;
		ExecOKDlg( QString("\"%1\" Loading Failure" ).arg( sSaverName ) );
		return false;
	}

	QWidget* pSaverWidget = iface->Init( LUSScreenSaverInterface::LUS_NORMAL );

	if ( pSaverWidget )
	{
		DBG_LOG( "LUSScreenSaver::ShowScreenSaver() > widget->showFullScreen()" );
		pSaverWidget->showFullScreen();
	}
	
	iface->Start();
	DBG_LOG( "LUSScreenSaver::ShowScreenSaver() > Module Start" );

	if ( ! IsAppLocked() )
	{
		PauseKeyHelper();
	}
	else
	{
		AuthDel();
	}

	return true;
}

void LUSScreenSaver::SetForceExecFlag()
{
	m_nForceExecuted = 1;
}

void LUSScreenSaver::SetScreenSaverMode( int nMode )
{
	if ( nMode < 0 )
	{
		m_nDisableSuspend = 200;
	}
	else
	{
		m_nDisableSuspend = nMode;
	}
}

int LUSScreenSaver::ExecOKDlg( const char* msg )
{
	return LUSScreenSaver::ExecBaseDlg(
					msg,
					QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape );
}

int LUSScreenSaver::ExecYesNoDlg( const char* msg )
{
	return LUSScreenSaver::ExecBaseDlg(
					msg,
					QMessageBox::Yes | QMessageBox::Default | QMessageBox::Escape,
					QMessageBox::No | QMessageBox::Escape );
}

int LUSScreenSaver::ExecBaseDlg( const char* msg, int button1, int button2 )
{
	QMessageBox mb( AppName(),
					msg,
					QMessageBox::NoIcon,
					button1,
					button2, QMessageBox::NoButton, TaskBarWidget() );
	mb.setIconPixmap( Resource::loadPixmap( "LUSScreenSaver_about" ) );
	mb.adjustSize();

	QPoint oDSize( QPEApplication::desktop()->width(),
					QPEApplication::desktop()->height() );
	QPoint oASize( mb.width(), mb.height() );
	mb.move( ( oDSize -  oASize ) / 2 );

	return mb.exec();
}

void LUSScreenSaver::VictimDlg()
{
	QMessageBox* mb = new QMessageBox( AppName(),
					"LCD OFF\n( if false to off, prease close this )",
					QMessageBox::NoIcon,
					QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
					QMessageBox::NoButton, QMessageBox::NoButton, TaskBarWidget() );
	mb->setIconPixmap( Resource::loadPixmap( "LUSScreenSaver_about" ) );
	mb->adjustSize();

	QPoint oDSize( QPEApplication::desktop()->width(),
					QPEApplication::desktop()->height() );
	QPoint oASize( mb->width(), mb->height() );
	mb->move( ( oDSize -  oASize ) / 2 );

	mb->show();
}

void LUSScreenSaver::AuthDone( int nID )
{
	DBG_LOG( QString( "LUSScreenSaver::AuthDone( %1 )" ).arg( nID ).latin1() );

	RestartKeyHelper();
	
	if ( ! m_pAuthDlg ) 
	{
		DBG_LOG( "LUSScreenSaver::AuthDone() > There is no LUSAuth Dialog." );
		return;
	}

	LUSScreenSaverAuthInterface* iface_auth;
	if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface_auth ) == QS_OK )
	{
		if ( iface_auth->Stop( nID ) )
		{
			iface_auth->release();
			m_pAuthDlg->unload();
			delete m_pAuthDlg;
			m_pAuthDlg = NULL;
			DBG_LOG( "LUSScreenSaver::AuthDone() > Authentication Done." );
		}
	}
	else
	{
		DBG_LOG( "LUSScreenSaver::AuthDone() > LUSAuth queryInterface() faild" );
		// for FATAL ERROR case
		m_pAuthDlg->unload();
		delete m_pAuthDlg;
		m_pAuthDlg = NULL;
		RestartKeyHelper();
	}
}

void LUSScreenSaver::AuthDel()
{
	DBG_LOG( "LUSScreenSaver::AuthDel()" );

	if ( ! m_pAuthDlg ) 
	{
		DBG_LOG( "LUSScreenSaver::AuthDell() > There is no LUSAuth Dialog." );
		return;
	}

	LUSScreenSaverAuthInterface* iface_auth;
	if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver, (QUnknownInterface**)&iface_auth ) == QS_OK )
	{
		iface_auth->Delete();
		iface_auth->release();
	}
	m_pAuthDlg->unload();
	delete m_pAuthDlg;
	m_pAuthDlg = NULL;

	DBG_LOG( "LUSScreenSaver::AuthDel() > Done." );
}

void LUSScreenSaver::RestartKeyHelper()
{
	DBG_LOG( "LUSScreenSaver::RestartKeyHelper()" );
	UnsetKeyFilter();
	QCopEnvelope* p_e = new QCopEnvelope( "QPE/KeyHelper", "restart()" );
	delete p_e;
}

void LUSScreenSaver::PauseKeyHelper()
{
	DBG_LOG( "LUSScreenSaver::PauseKeyHelper()" );
	QCopEnvelope* p_e = new QCopEnvelope( "QPE/KeyHelper", "pause()" );
	delete p_e;
	SetKeyFilter();
}

bool LUSScreenSaver::IsAppLocked()
{
	return (bool)m_pAuthDlg;
}

void LUSScreenSaver::DoLockedSuspend()
{
	if ( ! IsAppLocked() && IsLUSAuthInstalled() /* && IsPasswordEnable() */ )
	{
		DBG_LOG( "LUSScreenSaver::DoLockedSuspend() > Do Authentication" );
		LUSScreenSaverAuthInterface* iface_auth;
		m_pAuthDlg = new QLibrary( "lusauth", QLibrary::Immediately );
		if ( m_pAuthDlg->queryInterface( IID_LUSScreenSaver,
										(QUnknownInterface**)&iface_auth ) == QS_OK )
		{
			PauseKeyHelper();
			iface_auth->Start( rand(), TRUE );
			DBG_LOG( "LUSScreenSaver::DoLockedSuspend() > Do Suspend" );
			Global::execute( "suspend" );
		}
		else
		{
			DBG_LOG( "LUSScreenSaver::DoLockedSuspend() > LUSAuth quetyInterface() failed" );
			AuthDel();
			RestartKeyHelper();
		}
	}
}

