/*
	LUSScreenSaver
	Copyright (C) 2003-2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSScreenSaverMisc.h"
#include "LUSScreenSaverKeyFilter.h"
#include <sl/slmisc.h>

static QWidget*	g_pWidget		=	0;
static LUSKeyFilter* g_pKeyFilter = 0;

typedef struct _TAG_COUPLES{
	int nID;
	int nValue;
} COUPLES, *PCOUPLES;

QString& LUSMisc::AppName()
{
	static QString sAppName = "LUSScreenSaver";
	return sAppName;
}

void LUSMisc::InitWidget( QWidget* widget )
{
	g_pWidget = widget;
}

QWidget* LUSMisc::TaskBarWidget()
{
	return g_pWidget;
}

int LUSMisc::GetLightLevel()
{
	Config cfg( "qpe" );
	cfg.setGroup( "Screensaver" );

	if ( IsACOnline() )
	{
		return cfg.readNumEntry( "Brightness_AC", 255 );
	}
	else
	{
		return cfg.readNumEntry( "Brightness", 50 );
	}
}

int LUSMisc::GetLUSInterval()
{
	Config cfg( "qpe" );
	cfg.setGroup( "Screensaver" );

	if ( IsACOnline() )
	{
		return cfg.readNumEntry( "Interval_AC_ScreenSaver", 300 ) * 1000;
	}
	else
	{
		return cfg.readNumEntry( "Interval_Dim", 10 ) * 1000;
	}
}

int LUSMisc::GetDimLevel()
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Screensaver" );
	return cfg.readNumEntry( "Brightness_Dim", 1 );
}

void LUSMisc::SetLightLevel( const int bright )
{
	QCopEnvelope e( "QPE/System", "setBacklight(int)" );
	e << bright;
}

bool LUSMisc::IsACOnline()
{
	if ( PowerStatusManager::readStatus().acStatus()
								== PowerStatus::Online )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool LUSMisc::IsDimEnable()
{
	bool bRet;
	Config cfg( "qpe" );
	cfg.setGroup( "Screensaver" );
	bRet = cfg.readBoolEntry( "Dim" );
	
	return bRet;
}

bool LUSMisc::IsLightOffEnable()
{
	bool bRet;
	Config cfg( "qpe" );
	cfg.setGroup( "Screensaver" );
	bRet = cfg.readBoolEntry( "LightOff", FALSE );
	
	return bRet;
}

QString LUSMisc::GetSaverName()
{
	QString sName;
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Screensaver" );
	sName = cfg.readEntry( "Module" );
	
	return sName;
}

QString LUSMisc::AbsSaverPath( const QString& name )
{
	QString sPath = QString( "/mnt/card/QtPalmtop/plugins/screensaver/lib%1.so" ).arg( name );

	if ( ! QFile::exists( sPath ) )
	{
		sPath = QString( "/mnt/cf/QtPalmtop/plugins/screensaver/lib%1.so" ).arg( name );

		if ( ! QFile::exists( sPath ) )
		{
			sPath = QString( "/opt/QtPalmtop/plugins/screensaver/lib%1.so" ).arg( name );
		}
	}
	return sPath;
}

QString LUSMisc::GetSaverLibPath()
{
	QString sPath;
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Screensaver" );
	sPath = cfg.readEntry( "ModulePath" );
	
	return sPath;
}

int LUSMisc::GetIconMode()
{
	QString sPath;
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Misc" );
	return cfg.readNumEntry( "Icon", 1 );
}

void LUSMisc::SaveIconMode( int mode )
{
	QString sPath;
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Misc" );
	cfg.writeEntry( "Icon", mode );
}

bool LUSMisc::IsSuspendFirst()
{
	Config cfg( "qpe" );
	cfg.setGroup( "Screensaver" );
	
	int nSuspend, nScreenSave;
	if ( IsACOnline() )
	{
		int nIdx = cfg.readNumEntry( "Interval_AC_idx" );
		QString sIdxKey;
		sIdxKey.sprintf( "Interval_AC_%02d", nIdx );
		nSuspend = cfg.readNumEntry( sIdxKey );
		nScreenSave = cfg.readNumEntry( "Interval_AC_ScreenSaver" );
	}
	else
	{
		nSuspend = cfg.readNumEntry( "Interval" );
		nScreenSave = cfg.readNumEntry( "Interval_Dim" );
	}
	return ( nSuspend <= nScreenSave ) ? TRUE : FALSE;
}

bool LUSMisc::IsSpecialAppExists( int nMode )
{
	QString sProcesses;
	
	QDir procDir( "/proc" );
	procDir.setFilter( QDir::Dirs );
	const QFileInfoList *procList = procDir.entryInfoList();
	QFileInfoListIterator procIterator( *procList );
	QFileInfo *procInfo;
	while( (procInfo = procIterator.current()) )
	{
		QString procName = procInfo->absFilePath();
		procName.append("/status");
		if ( QFile::exists( procName ) )
		{
			FILE* fp;
			if ( ( fp = fopen( procName, "r" ) ) != 0 )
			{
				char name[101];
				fscanf( fp, "%*s %s", name );
				sProcesses.append( name );
				sProcesses.append( " " );

				fclose( fp );
			}
		}
		
		++ procIterator;
	}

	QStringList defApps;
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Misc" );
	QString sKey = nMode == LUS_SUSPEND ? "DisableSuspend" : "DisableScreenSaving";
	QString sKeyLine = cfg.readEntry( sKey );
	defApps = QStringList::split( ',', sKeyLine, FALSE );

	int nNumDef = defApps.count();

	bool bRet = FALSE;
	for ( int nCnt = 0; nCnt < nNumDef; nCnt ++ )
	{
		defApps[ nCnt ] = defApps[ nCnt ].stripWhiteSpace();
		if ( sProcesses.find( defApps[ nCnt ] ) != -1 )
		{
			bRet = TRUE;
			break;
		}
	}
	return bRet;
}

int LUSMisc::WhichIsFirst( int n0, int n1, int n2 )
{
	COUPLES Couples[3];
	PCOUPLES pRet;

	Couples[ 0 ].nID = n0;
	Couples[ 1 ].nID = n1;
	Couples[ 2 ].nID = n2;

	for ( int nCnt = 0; nCnt < 3; nCnt ++ )
	{
		Couples[ nCnt ].nValue = GetInterval( Couples[ nCnt ].nID );
		DBG_LOG( QString( "WhichIsFirst >> ID:%1 VALUE:%2" )
										.arg( Couples[ nCnt ].nID )
										.arg( Couples[ nCnt ].nValue ).latin1() );
	}
	
	pRet = &Couples[0];

	for ( int nCnt = 1; nCnt < 3; nCnt ++ )
	{
		if ( pRet->nValue >= Couples[ nCnt ].nValue )
		{
			pRet = &Couples[ nCnt ];
		}
	}

	DBG_LOG( QString( "WhichIsFirst >> RetID : %1" ).arg( pRet->nID ).latin1() );
	return pRet->nID;
}

int LUSMisc::GetInterval( int nKind )
{
	int nRet;
	Config cfg( "qpe" );
	cfg.setGroup( "Screensaver" );
	
	switch( nKind )
	{
	case LUS_SUSPEND:
		{
			if ( IsACOnline() )
			{
				int nIdx = cfg.readNumEntry( "Interval_AC_idx" );
				QString sIdxKey;
				sIdxKey.sprintf( "Interval_AC_%02d", nIdx );
				nRet = cfg.readNumEntry( sIdxKey );
			}
			else
			{
				nRet = cfg.readNumEntry( "Interval" );
			}
		}
		break;
	case LUS_SCREENSAVING:
		{
			if ( IsACOnline() )
			{
				nRet = cfg.readNumEntry( "Interval_AC_ScreenSaver" );
			}
			else
			{
				nRet = IsDimEnable() ? cfg.readNumEntry( "Interval_Dim" )
									: INT_MAX;
			}
		}
		break;
	case LUS_LIGHTOFF:
		{
			if ( IsACOnline() )
			{
				nRet = INT_MAX;
			}
			else
			{
				nRet = IsLightOffEnable() ? cfg.readNumEntry( "Interval_LightOff", INT_MAX )
											: INT_MAX;
			}
		}
		break;
	default:
		{
			nRet = INT_MAX;
		}
		break;
	}
	return nRet;
}

bool LUSMisc::IsPasswordEnable()
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Misc" );

	return cfg.readBoolEntry( "PasswordEnable", FALSE );
}

bool LUSMisc::IsLUSAuthInstalled()
{
	if ( QFile::exists( "/opt/QtPalmtop/lib/liblusauth.so" )
		|| QFile::exists( "/mnt/card/QtPalmtop/lib/liblusauth.so" )
		|| QFile::exists( "/mnt/cf/QtPalmtop/lib/liblusauth.so" ) )
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

void LUSMisc::SetKeyFilter()
{
	if ( ! g_pKeyFilter )
	{
		g_pKeyFilter = new LUSKeyFilter();
	}
	g_pKeyFilter->set();
	SlMisc::enableLockApplication( TRUE );
}

void LUSMisc::UnsetKeyFilter()
{
	if ( g_pKeyFilter )
	{
		g_pKeyFilter->unset();
		delete g_pKeyFilter;
		g_pKeyFilter = NULL;
	}
	SlMisc::enableLockApplication( FALSE );
}

