/*
	LUSScreenSaver
	Copyright (C) 2003-2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSScreenSaverWidget.h"
#include "LUSScreenSaverMisc.h"
#include "LUSScreenSaver.h"

using namespace LUSMisc;

static const char* _version_ =
#include "./version.txt"
;
// popup-menu item id
static const int LUS_MI_SETTINGS = 1;
static const int LUS_MI_ACTIVATE = 2;
static const int LUS_MI_LCDOFF   = 3;
static const int LUS_MI_ABOUT    = 4;

LUSScreenSaverWidget::LUSScreenSaverWidget( LUSScreenSaver* saver, QWidget* parent )
	: QWidget(parent,NULL)
{
	DBG_LOG( "LUSScreenSaverWidget::LUSScreenSaverWidget()" );
	m_pSaverImpl = saver;

	SetIconMode( GetIconMode() );

	m_pChannel = new QCopChannel( "QPE/LUSScreenSaver", this );
	connect( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );

	m_pSysChannel = new QCopChannel( "QPE/System", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveSysMessage( const QCString&, const QByteArray& ) ) );
}

LUSScreenSaverWidget::~LUSScreenSaverWidget()
{
	DBG_LOG( "LUSScreenSaverWidget::~LUSScreenSaverWidget()" );
	disconnect ( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	delete m_pChannel;

	delete m_pSysChannel;
}

void LUSScreenSaverWidget::receiveSysMessage( const QCString& msg, const QByteArray& data )
{
	if ( msg == "setScreenSaverMode(int)" )
	{
		int nMode;
		QDataStream stream(data, IO_ReadOnly);
		stream >> nMode;
		m_pSaverImpl->SetScreenSaverMode( nMode );
	}
	else if ( msg == "appForeground(QString)" || msg == "resumed()" )
	{
		if ( m_pSaverImpl->IsAppLocked() )
		{
			SetKeyFilter();
		}
	}
}

void LUSScreenSaverWidget::receiveMessage( const QCString& msg, const QByteArray& data )
{
	QDataStream stream(data, IO_ReadOnly);
	if ( msg == "SetScreenSaverActivate()" )
	{
		SetScreenSaverActivate();
	} 
	else if ( msg == "AuthDone(int)" )
	{
		int nID;
		stream >> nID;
		AuthDone( nID );
	}
	else if ( msg == "LockedSuspend()" )
	{
		m_pSaverImpl->DoLockedSuspend();
	}
	else if ( msg == "SetLCDBlank()")
	{
		SetLCDBlank();
	}
	else if ( msg == "ShowIcon(int)")
	{
		int mode;
		stream >> mode;
		SetIconMode( mode );
		repaint();
	}
	else if ( msg == "enable()")
	{
		;
	}
	else if ( msg == "disable()")
	{
		;
	}
	else if ( msg == "Version()")
	{
		Version();
	}
}

void LUSScreenSaverWidget::SetIconMode( int mode )
{
	DBG_LOG( "LUSScreenSaverWidget::SetIconMode()" );
	if ( mode == 1 )
	{
		m_oPixmap = Resource::loadPixmap( "LUSScreenSaver_icon" );
	}
	else
	{
		m_oPixmap = QPixmap();
	}

	setFixedSize( m_oPixmap.size() );
	SaveIconMode( mode );
}

void LUSScreenSaverWidget::Version()
{
	QString ver = AppName();
	ver.append( _version_ );
	Global::statusMessage( ver );
}

void LUSScreenSaverWidget::SetScreenSaverActivate()
{
	QWSServer::setScreenSaverInterval( 800 );
	m_pSaverImpl->SetForceExecFlag();
}

void LUSScreenSaverWidget::mouseReleaseEvent( QMouseEvent * )
{
	if ( m_pSaverImpl->IsAppLocked() )
	{
		return;
	}
	
	QPopupMenu* pPopupMenu = new QPopupMenu( this );

	pPopupMenu->insertItem( tr( " Settings " ), LUS_MI_SETTINGS );
	pPopupMenu->insertItem( tr( " Activate " ), LUS_MI_ACTIVATE );
	pPopupMenu->insertItem( tr( " LCD Off  " ), LUS_MI_LCDOFF   );
	pPopupMenu->insertSeparator();
	pPopupMenu->insertItem( tr( " about .. " ), LUS_MI_ABOUT    );

	connect( pPopupMenu, SIGNAL( activated( int ) ),
								this, SLOT( MenuAction( int ) ) );

	QPoint oPos = mapToGlobal( QPoint( 0, 0 ) );
	QSize sizeMenu = pPopupMenu->sizeHint();
	pPopupMenu->popup( oPos - QPoint( ( sizeMenu.width() - width() ) / 2,
													sizeMenu.height() ) );
}

void LUSScreenSaverWidget::MenuAction( int nId )
{
	switch ( nId )
	{
	case LUS_MI_SETTINGS:
		ExecSettingsDialog();
		break;
	case LUS_MI_ACTIVATE:
		SetScreenSaverActivate();
		break;
	case LUS_MI_LCDOFF:
		SetLCDBlank();
		break;
	case LUS_MI_ABOUT:
		m_pSaverImpl->ExecOKDlg( QString(
											"Less UnSaticefactory\n"\
											" ScreenSaver Ver. %1\n"\
											" (C) 2003  M.Ono\n"\
											"            GPL Lisenced"\
										 )
										.arg( _version_ ).latin1() );
		break;
	default:
		break;
	}
}

void LUSScreenSaverWidget::ExecSettingsDialog()
{
	Global::execute( "LUSScreenSaverUtil" );
}

void LUSScreenSaverWidget::SetLCDBlank()
{
	m_pSaverImpl->VictimDlg();
	QCopEnvelope e( "QPE/System", "setBlankLCD(int)" );
	e << 1;
}

void LUSScreenSaverWidget::paintEvent(QPaintEvent *)
{
	SetIconMode( GetIconMode() );
	QPainter p(this);
	p.drawPixmap(0, 0, m_oPixmap);
}

void LUSScreenSaverWidget::AuthDone( int nID )
{
	m_pSaverImpl->AuthDone( nID );
}
